--[[ 
    Button Framework - Dylan Coakley
    Version 0.2

    Default Button Example

    ButtonsOnScreen = {}
    b1 = Button:new("Save", 50, 50, 100, 100, "Save")
    b1.colour.r, b1.colour.g, b1.colour.b, b1.colour.a  = 100, 100, 100, 255
    
    table.insert(ButtonsOnScreen, b1) -- Save Button
]]--

Button = {}

function Button:new(screen, action, x, y, w, h, text)
    local ButtonO = {
        screen = screen,
    	action = action,
        x = x,
        y = y,
        w = w,
        h = h,
        colour = { r = 0, g = 0, b = 0, a = 0 },
        borderColour = { r = 0, g = 0, b = 0, a = 0 },
        borderSize = 10,
        textColour = { r = 255, g = 255, b = 255, a = 255 },
        image = "",
        text = text,
        showImage = false,
        showBorder = false,
        showColour = true,
        showText = true,
        fh = font:getHeight(),
        fw = font:getWidth(text),
        draw = false
    }
    return ButtonO
end

function Button:draw()
    for i = 1, #ButtonsOnScreen do
        local btn = ButtonsOnScreen[i]

        if btn.draw == true then
            if btn.showColour == true then
                love.graphics.setColor(btn.colour.r, btn.colour.g, btn.colour.b, btn.colour.a)
                love.graphics.rectangle("fill",btn.x, btn.y, btn.w, btn.h)
            end
            if btn.showBorder == true then
                love.graphics.setColor(btn.borderColour.r, btn.borderColour.g, btn.borderColour.b, btn.borderColour.a)
                love.graphics.rectangle("fill",btn.x+btn.borderSize, btn.y+btn.borderSize, btn.w-(btn.borderSize*2), btn.h-(btn.borderSize*2))
            end
            if btn.showImage == true then
                love.graphics.draw(btn.image, btn.x, btn.y)
            end
            if btn.showText == true then
                love.graphics.setColor(btn.textColour.r, btn.textColour.g, btn.textColour.b, btn.textColour.a)
                love.graphics.print(btn.text, (btn.x+btn.w/2)-btn.fw/2, (btn.y+btn.h/2)-btn.fh/2)
            end
            love.graphics.reset()
        end
    end
end

function Button:mousepressed(x, y)
	for i = 1, #ButtonsOnScreen do
		local btn = ButtonsOnScreen[i]
		if x > btn.x and x < btn.x+btn.w and y > btn.y and y < btn.y+btn.h then
            runAction(btn.screen, btn.action)
	  	end
	end
end
