--[[
	WhiteGuardian v0.2.6
	Author: Dylan Coakley
]]--

local sti = require 'libs.STI'
require('libs/AnAL')
lady = require 'libs/lady'
require 'classes/Player'
require 'classes/Item'
require 'classes/Menu'
require 'classes/Skill'
require 'classes/Spell'
require 'classes/framework/Button'

font = love.graphics.newFont(12)
smallfont = love.graphics.newFont( 14 )
largefont = love.graphics.newFont( 18 )

-- Simple box collision check between two rectangles A and B
function box_collision(xa, ya, wa, ha, xb, yb, wb, hb)
	return ((xa <= xb + wb) and (xb <= xa + wa) and (ya <= yb + hb) and (yb <= ya + ha))
end

function checkUpCollision(item)
	if box_collision(player.x, player.y-5, player.w, player.h, item.x, item.y, item.w, item.h) and item.itemType == 100 then
		collisionUp = true
	else
		collisionUp = false
	end
end

function checkDownCollision(item)
	if box_collision(player.x, player.y+5, player.w, player.h, item.x, item.y, item.w, item.h) and item.itemType == 100 then
		collisionDown = true
	else
		collisionDown= false
	end
end

function checkLeftCollision(item)
	if box_collision(player.x-5, player.y, player.w, player.h, item.x, item.y, item.w, item.h) and item.itemType == 100 then
		collisionLeft= true
	else
		collisionLeft = false
	end
end

function checkRightCollision(item)
	if box_collision(player.x+5, player.y, player.w, player.h, item.x, item.y, item.w, item.h) and item.itemType == 100 then
		collisionRight = true
	else
		collisionRight = false
	end
end

function maketransparent(file)
   local imgdata = love.image.newImageData(file)
   for i=0,imgdata:getWidth()-1 do
      for j=0,imgdata:getHeight()-1 do
         local r,g,b,a = imgdata:getPixel(i,j)
         if r == 43 and g == 133 and b == 133 and a == 255 then
            imgdata:setPixel(i,j,255,255,255,0)
         end
      end
   end
   return love.graphics.newImage(imgdata)
end

function addToInv(itemToAdd)
    local addToSlot = #inventory + 1
    for i = 1, #inventory do
        if inventory[i].name == itemToAdd.name then
            addToSlot = i
            inventory[addToSlot].amount = inventory[addToSlot].amount + itemToAdd.amount
            return
        end
    end
    table.insert(inventory, itemToAdd)
end

function getCoordsByPercent(percentW, percentH)
	width, height = love.graphics.getDimensions( )
	w = (width/100)*percentW
	h = (height/100)*percentH
	return w,h
end

function getCoordsByPercentX(percentW)
	width, height = love.graphics.getDimensions( )
	w = (width/100)*percentW
	return w
end

function getCoordsByPercentY(percentH)
	width, height = love.graphics.getDimensions( )
	h = (height/100)*percentH
	return h
end

function runAction(screen, action)
	print('screen '..screen..' state '..state..' action '..action)
	if screen == state and action == 'StartGame' then
		print('Execute startGame action')
		startGame = true
	end
	if screen == state and action == 'Save' then
		print('Execute Save action')
	end
	if screen == state and action == 'Load' then
		print('Execute Load action')
	end
end

function setupGame()
	-- Setup Screenobjects
    ButtonsOnScreen = {}

    buttonStartGame = Button:new('Start','StartGame', 320, 190, 150, 100, 'New Game')
    buttonStartGame.colour.r, buttonStartGame.colour.g, buttonStartGame.colour.b, buttonStartGame.colour.a  = 60, 60, 60, 255
    buttonStartGame.borderColour.r, buttonStartGame.borderColour.g, buttonStartGame.borderColour.b, buttonStartGame.borderColour.a  = 233, 198, 175, 255
    buttonStartGame.borderSize = 2
    buttonStartGame.textColour.r, buttonStartGame.textColour.g, buttonStartGame.textColour.b, buttonStartGame.textColour.a  = 60, 60, 60, 255
    buttonStartGame.showBorder = true
    buttonStartGame.draw = true
    table.insert(ButtonsOnScreen, buttonStartGame)

    buttonSave = Button:new('PauseMenu_tab5','Save', 300, 170, 100, 100, 'Save')
    buttonSave.colour.r, buttonSave.colour.g, buttonSave.colour.b, buttonSave.colour.a  = 60, 60, 60, 255
    buttonSave.borderColour.r, buttonSave.borderColour.g, buttonSave.borderColour.b, buttonSave.borderColour.a  = 233, 198, 175, 255
    buttonSave.borderSize = 2
    buttonSave.textColour.r, buttonSave.textColour.g, buttonSave.textColour.b, buttonSave.textColour.a  = 60, 60, 60, 255
    buttonSave.showBorder = true
    table.insert(ButtonsOnScreen, buttonSave) -- Save Button mainMenu 5

    buttonLoad = Button:new('PauseMenu_tab5','Load', 500, 170, 100, 100, 'Load')
    buttonLoad.colour.r, buttonLoad.colour.g, buttonLoad.colour.b, buttonLoad.colour.a  = 60, 60, 60, 255
    buttonLoad.borderColour.r, buttonLoad.borderColour.g, buttonLoad.borderColour.b, buttonLoad.borderColour.a  = 233, 198, 175, 255
    buttonLoad.borderSize = 2
    buttonLoad.textColour.r, buttonLoad.textColour.g, buttonLoad.textColour.b, buttonLoad.textColour.a  = 60, 60, 60, 255
    buttonLoad.showBorder = true
    table.insert(ButtonsOnScreen, buttonLoad) -- Load Button mainMenu 5

	newGame = true
	showConsole = false

	selectedP = 'jake'
	selectedPClass = 'Knight'

	-- Create the player
	plrX, plrY = getCoordsByPercent(50,50)

	if newGame == true then
		inventory = {}
		player = Player:new(plrX-16, plrY-24, 32, 50, 'Jake',selectedP,selectedPClass, 20, 10)
		else
		player = lady.load_all('savefile')
		inventory = lady.load_all('inventory')
	end

	party = {}
	table.insert(party, player)
	skill1 = Skill:new('Attack',2)
	skill2 = Skill:new('Defense',10)
	skill3 = Skill:new('MAttack',1)
	skill4 = Skill:new('MDefense',2)
	skill5 = Skill:new('Critial',7)
	skill6 = Skill:new('Luck',5)
	skill7 = Skill:new('Range Atk',2)
	skill8 = Skill:new('Strength',10)
	skill9 = Skill:new('Aura',1)
	skill10 = Skill:new('Vitality',1)

	table.insert(party[1].skills, skill1)
	table.insert(party[1].skills, skill2)
	table.insert(party[1].skills, skill3)
	table.insert(party[1].skills, skill4)
	table.insert(party[1].skills, skill5)
	table.insert(party[1].skills, skill6)
	table.insert(party[1].skills, skill7)
	table.insert(party[1].skills, skill8)
	table.insert(party[1].skills, skill9)
	table.insert(party[1].skills, skill10)

	spell1 = Spell:new('Healing Shrine', 15, 'heal')
	table.insert(party[1].spells, spell1)

	itemw, itemh = getCoordsByPercent(50,50)

	-- Create the items on the map
	itemsOnMap = {
	-- params: x,y,w,h,itemID,itemName,fileName,atk,def,specialPercent,specialType, itemAmount, itemType
		Item:new(itemw,itemh,32, 32, 1, 'Wooden Bat','woodenBat',2,nil, 0, '', 1, 1),
		Item:new(300,50,32, 32, 2, 'Vital Ring','vitalRing',nil,nil, 5, 'Health', 1, 7),
		Item:new(70,250,32, 32, 3, 'Rusty Dagger', 'rustyIronDagger',2,nil, 0, '', 1, 1),
		Item:new(60,110,32, 32, 1, 'bronzeSpear','bronzeSpear',2,nil, 0, '', 1, 1),
		Item:new(390,70,32, 32, 2, 'Cloth Robe','clothRobe',nil,1, 0, '', 1, 5),
		Item:new(100,250,32, 32, 3, 'Pine Plank Shield', 'pinePlankShield',nil,2, 0, '', 1, 3),
		Item:new(80,100,32, 32, 1, 'Wooden Bat','woodenBat',2,nil, 0, '', 1, 1),
		Item:new(350,50,32, 32, 2, 'Ox Ring','oxRing',2,nil, 2, 'Strength', 1, 8),
		Item:new(90,250,32, 32, 3, 'Cloth Robe', 'clothRobe',2,nil, 0, '', 1, 5),
		Item:new(20,110,32, 32, 1, 'bronzeSpear','bronzeSpear',2,nil, 0, '', 1, 1),
		Item:new(10,70,32, 32, 2, 'Cloth Hood','clothHood',nil,1, 0, '', 1, 2),
		Item:new(50,250,32, 32, 3, 'Cotton Boots', 'cottonBoot',nil,2, 0, '', 1, 4),
		Item:new(50,250,32, 32, 3, 'Vital Necklace', 'vitalNeck',nil,2, 7, 'Health', 1, 6),
		Item:new(402,110,220, 130, 3, 'House', '',nil,2, 7, '', 1, 100)
	}

	-- Setup item images here to reduce lag
	itemImages = {}
	for i = 1, #itemsOnMap do
		local item = itemsOnMap[i]
		if item.image ~= 'assets/items/small/.png' then
			itemImages[i] = love.graphics.newImage(item.image)
		end
	end

	i		= 1
	speed	= 256

	-- Load multiple maps
	maps = {
		sti.new('assets/maps/rpgmap.lua')
	}

	col = {}

	for k=1, #maps do
		local layer = 2

		-- Add a Custom Layer
		maps[k]:addCustomLayer('Sprite Layer', layer)
		local spriteLayer = maps[k].layers['Sprite Layer']
		local image = maketransparent(player.image)

		standing = newAnimation(image, 128, 192, 0.1, -1)
		standing:addFrame(0, 0, 32, 48, 0.1)

		walkLeft = newAnimation(image, 128, 192, 0.1, -1)
		walkLeft:addFrame(0, 48, 32, 48, 0.1)
		walkLeft:addFrame(32, 48, 32, 48, 0.1)
		walkLeft:addFrame(64, 48, 32, 48, 0.1)
		walkLeft:addFrame(96, 48, 32, 48, 0.1)

		walkRight = newAnimation(image, 128, 192, 0.1, -1)
		walkRight:addFrame(0, 96, 32, 48, 0.1)
		walkRight:addFrame(32, 96, 32, 48, 0.1)
		walkRight:addFrame(64, 96, 32, 48, 0.1)
		walkRight:addFrame(96, 96, 32, 48, 0.1)

		walkDown = newAnimation(image, 128, 192, 0.1, -1)
		walkDown:addFrame(0, 0, 32, 48, 0.1)
		walkDown:addFrame(32, 0, 32, 48, 0.1)
		walkDown:addFrame(64, 0, 32, 48, 0.1)
		walkDown:addFrame(96, 0, 32, 48, 0.1)

		walkUp = newAnimation(image, 128, 192, 0.1, -1)
		walkUp:addFrame(0, 144, 32, 48, 0.1)
		walkUp:addFrame(32, 144, 32, 48, 0.1)
		walkUp:addFrame(64, 144, 32, 48, 0.1)
		walkUp:addFrame(96, 144, 32, 48, 0.1)

		-- Add Custom Data
		spriteLayer.sprite = {
			--image = love.graphics.newImage('assets/sprites/'..sprite..'.png'),  this draws the whole spritesheet, not what I need
			x = 495,
			y = 462,
			r = 0,
		}
	end
	Menu:load()

	-- Setup Lady
	lady.register_class(player, 'player')
	lady.register_resource(player.image, player.fileName)

	--state = 'main'

end

function love.load()

	startGame = false
	state = 'Start'
	setupGame()

	gameStartBackgroundImage = love.graphics.newImage('assets/menu/gameMainMenu.png')

end

function love.update(dt)
if startGame == true then
	-- Check for collisions between the player and the items on the map
	for i = 1, #itemsOnMap do
		local item = itemsOnMap[i]
		if item.health > 0 then
			-- If the player collide with an item on the map decrease health of item
			if box_collision(player.x, player.y, player.w, player.h, item.x, item.y, item.w, item.h) and item.itemType ~= 100 then
				itemsOnMap[i].health = itemsOnMap[i].health - 1
				addToInv(itemsOnMap[i])
			end
			checkUpCollision(itemsOnMap[i])
			checkDownCollision(itemsOnMap[i])
			checkLeftCollision(itemsOnMap[i])
			checkRightCollision(itemsOnMap[i])
		end
	end

	s_w,s_h =  love.graphics.getDimensions( )
	-- Cap speed
	if speed < 128	then speed = 128 end
	if speed > 512	then speed = 512 end

	-- Update sprite's coordinates
	sprite = maps[i].layers['Sprite Layer'].sprite
	down = love.keyboard.isDown

	if showMenu == false then
		if down('w') or down('up') and collisionUp== false then
			-- Update positions for items on map
			for i = 1, #itemsOnMap do
				if itemsOnMap[i].health > 0 then
					itemsOnMap[i].y =  itemsOnMap[i].y + speed * dt
				end
			end
			sprite.y = sprite.y - speed * dt
			walkDown:stop()
			walkUp:play()
			walkUp:update(dt)
		end
		if down('s') or down('down') and collisionDown== false then
			-- Update positions for items on map
			for i = 1, #itemsOnMap do
				if itemsOnMap[i].health > 0 then
					itemsOnMap[i].y =  itemsOnMap[i].y - speed * dt
				end
			end
			sprite.y = sprite.y + speed * dt
			walkUp:stop()
			walkDown:play()
			walkDown:update(dt)
		end
		if down('a') or down('left') and collisionLeft == false then
			-- Update positions for items on map
			for i = 1, #itemsOnMap do
				if itemsOnMap[i].health > 0 then
					itemsOnMap[i].x =  itemsOnMap[i].x + speed * dt
				end
			end
			sprite.x = sprite.x - speed * dt
			walkRight:stop()
			walkLeft:play()
			walkLeft:update(dt)
		end
		if down('d') or down('right') and collisionRight == false then
			-- Update positions for items on map
			for i = 1, #itemsOnMap do
				if itemsOnMap[i].health > 0 then
					itemsOnMap[i].x =  itemsOnMap[i].x - speed * dt
				end
			end
			sprite.x = sprite.x + speed * dt
			walkLeft:stop()
			walkRight:play()
			walkRight:update(dt)
		 end

		for _, map in pairs(maps) do
			map:update(dt)
		end
	end

end
end

function love.draw()

if startGame == true then
	local sprite = maps[i].layers['Sprite Layer'].sprite
	local ww = love.graphics.getWidth()
	local wh = love.graphics.getHeight()
	local tx = math.floor(-sprite.x + ww / 2 - 16)
	local ty = math.floor(-sprite.y + wh / 2 - 16)

	-- Draw sprite in centre of screen
	love.graphics.push()
	love.graphics.translate(tx, ty)
	maps[i]:setDrawRange(tx, ty, ww, wh)
	maps[i]:draw()
	--maps[i]:drawCollisionMap(col[i])
	love.graphics.pop()
	x = (s_w/2) - 16
	y = (s_h/2) - 24

	--player.x, player.y = sprite.x,sprite.y

	-- Draw the items on the map if not collected
	for i = 1, #itemsOnMap do
		local item = itemsOnMap[i]
		if item.health > 0 and item.image ~= 'assets/items/small/.png' then
			love.graphics.draw(itemImages[i], item.x, item.y)
		end
	end
	-- Player movement animations
	if down('a') or down('left') then
		walkLeft:draw(x, y)
	end
	if down('d') or down('right') then
		walkRight:draw(x, y)
	end
	if down('s') or down('down') then
		walkDown:draw(x, y)
	end
	if down('w') or down('up') then
		walkUp:draw(x, y)
	end
	if ((down('a') or down('left'))==false) and ((down('d') or down('right'))==false) and ((down('s') or down('down'))==false) and ((down('w') or down('up'))==false) then
		standing:draw(x, y)
	end

	-- Setup Menu functionality
	Menu:draw()

	love.graphics.setColor(255, 255, 255)
	if showConsole == true then
		mouseX, mouseY = love.mouse.getPosition()
		love.graphics.print('Current FPS: '..tostring(love.timer.getFPS( )), 10, 10)
		love.graphics.print(' x: '..tostring(player.x)..' y: '..tostring(player.y), 10, 30)
		love.graphics.print(' mouseX: '..tostring(mouseX)..' mouseY: '..tostring(mouseY), 10, 50)

		-- Draw the bounding rectangles for the items on the map
		for i = 1, #itemsOnMap do
			local item = itemsOnMap[i]
			if item.health > 0 then
				love.graphics.rectangle('line',item.x, item.y, item.w, item.h)
			end
		end

		love.graphics.rectangle('line',player.x, player.y, player.w, player.h)
	end

else
	love.graphics.draw(gameStartBackgroundImage, 0, 0)
	Button:draw()
	end
end

function love.keypressed(key, isrepeat)
	if key == 'tab' then
		if showConsole == false then
			showConsole = true
		else
			showConsole = false
		end
	end
	if key == 'escape' then
		love.event.quit()
	end
	Menu:keypressed(key, isrepeat)
end

function love.mousepressed(x, y, button)
   Menu:mousepressed(x, y, button)
   Button:mousepressed(x, y)
end

function love.resize(w, h)
	for k=1, #maps do
		maps[k]:resize(w, h)
	end
	showMenu = false
end